from mock import patch
from test import BaseTest
import rtvedownloader
import sys
import unittest

class ParserTest(BaseTest):
    
    def setUp(self):
        BaseTest.setUp(self)
        self.argv = sys.argv
        # self.patcher = patch('rtvedownloader.RTVEDownloader')
        # self.patched_instance = self.patcher.start()

    
    def test_main_method_without_parameters(self):
        args = ['rtvedownload']
        sys.argv = args
        
        try:
            rtvedownloader.main()
        except SystemExit, e:
            self.assertEquals(type(e), type(SystemExit()))
            self.assertEquals(e.code, 1)
            
        self.assert_sysout_has('No url defined')

    @patch('rtvedownloader.RTVEDownloader.download')
    def test_main_method_with_no_action_parameter_and_some_url(self, download_mock):
        download_mock.return_value = (True, 'file downloaded')
        
        args = ['rtvedownload', '-v', '-n', self.url]
        sys.argv = args
        try:
            rtvedownloader.main()
        except SystemExit, e:
            self.assertEquals(type(e), type(SystemExit()))
            self.assertEquals(e.code, 0)
        
        download_mock.assert_called_once_with(self.url)
        self.assert_sysout_has('file downloaded')


    @patch('rtvedownloader.RTVEDownloader.download')
    def test_main_method_returning_false(self, download_mock):
        download_mock.return_value = (False, 'file could not be downloaded')
        
        args = ['rtvedownload', '-v', self.url]
        sys.argv = args
        try:
            rtvedownloader.main()
        except SystemExit, e:
            self.assertEquals(type(e), type(SystemExit()))
            self.assertEquals(e.code, 0)
        
        download_mock.assert_called_once_with(self.url)
        self.assert_sysout_has('file could not be downloaded')
        self.assert_sysout_has(self.url)


    @patch('rtvedownloader.RTVEDownloader.download')
    def test_main_method_throws_exception(self, download_mock):
        download_mock.return_value = (False, '')
        
        def side_effect(*args, **kwargs):
            raise Exception('Raised exception')
            
        download_mock.side_effect = side_effect
        
        args = ['rtvedownload', '-v', self.url]
        sys.argv = args
        try:
            rtvedownloader.main()
        except SystemExit, e:
            self.assertEquals(type(e), type(SystemExit()))
            self.assertEquals(e.code, 0)
        
        download_mock.assert_called_once_with(self.url)
        self.assert_sysout_has('Unexpected exception')
        
        
    def test_main_method_fails_with_filename_and_two_urls(self):
        args = ['rtvedownload', '-v', '-f', 'some_filename', self.url, self.url]
        sys.argv = args
        
        try:
            rtvedownloader.main()
        except SystemExit, e:
            self.assertEquals(type(e), type(SystemExit()))
            self.assertEquals(e.code, 1)
            
        self.assert_sysout_has('Could not set filename')
        
        
    def tearDown(self):
        BaseTest.tearDown(self)
        sys.argv = self.argv
        # self.patcher.stop()
        
        
if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()